# {{ package.name }}
====================
## Packages:{% for child in package.children recursive %}
- {{ child.name }}
{{ child.documentation }}
### Classes:{% for cls in child.classes %}
- {{ cls.name }}{% endfor %}
{% if child.children %}{{ loop(child.children) }}{% endif %}
{% endfor %}
